/*
    Crystal Space Shared String Vector class
    Copyright (C) 1998,1999 by Andrew Zabolotny <bit@eltech.ru>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CS_SCFSTR_H__
#define __CS_SCFSTR_H__

#include "iutil/string.h"
#include "csutil/csstring.h"

/// This is a thin SCF wrapper around csString
class scfString : public iString
{
  csString s;

public:
  SCF_DECLARE_IBASE;

  /// Create an empty scfString object
  scfString ()
  { SCF_CONSTRUCT_IBASE (0); }

  /// Create an scfString object and reserve space for iLength characters
  scfString (size_t iLength) : s(iLength)
  { SCF_CONSTRUCT_IBASE (0); }

  /// Copy constructor
  scfString (const iString &copy) : s(copy.GetData())
  { SCF_CONSTRUCT_IBASE (0); }

  /// Yet another copy constructor
  scfString (const char *copy) : s(copy)
  { SCF_CONSTRUCT_IBASE (0); }

  /// Destroy a scfString object
  virtual ~scfString () {}

  /// Get the pointer to the internal csString.
  const csString& GetCsString () const { return s; }

  /// Get the pointer to the internal csString.
  csString& GetCsString () { return s; }

  /// Set string capacity to NewSize characters.
  virtual void SetCapacity (size_t NewSize);
  /// Get string capacity.
  virtual size_t GetCapacity() const;
  /// Set the allocation growth increment.
  virtual void SetGrowsBy(size_t);
  /// Get the allocation growth increment.
  virtual size_t GetGrowsBy() const;
  /// Set exponential allocation growth.  Overrides SetGrowsBy().
  virtual void SetGrowsExponentially(bool);
  /// Get exponential growth setting.
  virtual bool GetGrowsExponentially() const;

  /// Truncate the string
  virtual void Truncate (size_t iPos);

  /// Set string maximal capacity to current string length
  virtual void Reclaim ();

  /// Get a copy of this string
  virtual iString *Clone () const;

  /// Get a pointer to ASCIIZ string
  virtual char *GetData () const;

  /// Query string length
  virtual size_t Length () const;

  /// Get a reference to iPos'th character
  virtual char& operator [] (size_t iPos);

  /// Set characetr number iPos to iChar
  virtual void SetAt (size_t iPos, char iChar);

  /// Get character at position iPos
  virtual char GetAt (size_t iPos) const;

  /// Insert another string into this one at position iPos
  virtual void Insert (size_t iPos, iString *iStr);

  /// Overlay another string onto a part of this string
  virtual void Overwrite (size_t iPos, iString *iStr);

  /// Append an ASCIIZ string to this one (up to iCount characters)
  virtual iString &Append (const char *iStr, size_t iCount = (size_t)-1);

  /// Append a string to this one (possibly iCount characters from the string)
  virtual iString &Append (const iString *iStr, size_t iCount = (size_t)-1);

  /**
   * SubString another string out of this one. The result is placed
   * in 'sub'. The substring is from 'start', of length 'len'.
   */
  virtual void SubString (iString * sub, size_t start, size_t len);

  /**
   * Find first character 'c' from position 'p'.
   * If the character cannot be found, this function returns (size_t)-1
   */
  virtual size_t FindFirst (const char c, size_t p = (size_t)-1);
  /**
   * Find first character 'c', counting backwards from position 'p'.
   * Default position is the end of the string.
   * If the character cannot be found, this function returns (size_t)-1
   */
  virtual size_t FindLast (const char c, size_t p = (size_t)-1);

  /// Format.
  virtual void Format (const char *format, ...) CS_GNUC_PRINTF (2, 3);
  /// Format.
  virtual void FormatV (const char *format, va_list args);

  /// Replace contents of this string with the contents of another
  virtual void Replace (const iString *iStr, size_t iCount = (size_t)-1);

  /// Check if two strings are equal
  virtual bool Compare (const iString *iStr) const;

  /// Compare two strings ignoring case
  virtual bool CompareNoCase (const iString *iStr) const;
};

#endif // __CS_SCFSTR_H__
